@@ -371,7 +371,8 @@ _pygi_closure_convert_arguments (PyGIInvokeState *state,
     for (i = 0; i < _pygi_callable_cache_args_len (cache); i++) {
         PyGIArgCache *arg_cache = g_ptr_array_index (cache->args_cache, i);
 
-        if (arg_cache->direction & PYGI_DIRECTION_FROM_PYTHON) {
+        if (arg_cache->direction & PYGI_DIRECTION_FROM_PYTHON &&
+                state->arg_values[i].v_pointer) {
             state->arg_pointers[i].v_pointer = state->arg_values[i].v_pointer;
             state->arg_values[i] = *(GIArgument *) state->arg_values[i].v_pointer;
         }
@@ -551,6 +552,13 @@ _pygi_closure_handle (ffi_cif *cif,
     gboolean success;
     PyGIInvokeState state = { 0, };
 
+    /* Ignore closures when Python is not initialized. This can happen in cases
+     * where calling Python implemented vfuncs can happen at shutdown time.
+     * See: https://bugzilla.gnome.org/show_bug.cgi?id=722562 */
+    if (!Py_IsInitialized()) {
+        return;
+    }
+
     /* Lock the GIL as we are coming into this code without the lock and we
       may be executing python code */
     py_state = PyGILState_Ensure ();
