@@ -33,6 +33,12 @@ else:
     UNICHAR = "♥"
 
 
+const_str = b'const \xe2\x99\xa5 utf8'
+if sys.version_info >= (3, 0):
+    const_str = const_str.decode('UTF-8')
+noconst_str = 'non' + const_str
+
+
 class RawGList(ctypes.Structure):
     _fields_ = [('data', ctypes.c_void_p),
                 ('next', ctypes.c_void_p),
@@ -221,27 +227,34 @@ class TestEverything(unittest.TestCase):
         timeout = v.lookup_value('timeout', None)
         self.assertEqual(timeout.get_int32(), 10)
 
-    def test_string(self):
-        const_str = b'const \xe2\x99\xa5 utf8'
-        if sys.version_info >= (3, 0):
-            const_str = const_str.decode('UTF-8')
-        noconst_str = 'non' + const_str
-
+    def test_utf8_const_return(self):
         self.assertEqual(Everything.test_utf8_const_return(), const_str)
+
+    def test_utf8_nonconst_return(self):
         self.assertEqual(Everything.test_utf8_nonconst_return(), noconst_str)
+
+    def test_utf8_out(self):
         self.assertEqual(Everything.test_utf8_out(), noconst_str)
 
+    def test_utf8_const_in(self):
         Everything.test_utf8_const_in(const_str)
+
+    def test_utf8_inout(self):
         self.assertEqual(Everything.test_utf8_inout(const_str), noconst_str)
 
+    def test_filename_return(self):
         self.assertEqual(Everything.test_filename_return(), ['åäö', '/etc/fstab'])
 
+    def test_int_out_utf8(self):
         # returns g_utf8_strlen() in out argument
         self.assertEqual(Everything.test_int_out_utf8(''), 0)
         self.assertEqual(Everything.test_int_out_utf8('hello world'), 11)
         self.assertEqual(Everything.test_int_out_utf8('åäö'), 3)
 
+    def test_utf8_out_out(self):
         self.assertEqual(Everything.test_utf8_out_out(), ('first', 'second'))
+
+    def test_utf8_out_nonconst_return(self):
         self.assertEqual(Everything.test_utf8_out_nonconst_return(), ('first', 'second'))
 
     def test_enum(self):
@@ -356,53 +369,88 @@ class TestEverything(unittest.TestCase):
         # test that there are no duplicates returned
         self.assertEqual(len(attr_list), len(set(attr_list)))
 
-    def test_array(self):
+    def test_array_int_in_empty(self):
         self.assertEqual(Everything.test_array_int_in([]), 0)
+
+    def test_array_int_in(self):
         self.assertEqual(Everything.test_array_int_in([1, 5, -2]), 4)
+
+    def test_array_int_out(self):
         self.assertEqual(Everything.test_array_int_out(), [0, 1, 2, 3, 4])
+
+    def test_array_int_full_out(self):
         self.assertEqual(Everything.test_array_int_full_out(), [0, 1, 2, 3, 4])
+
+    def test_array_int_none_out(self):
         self.assertEqual(Everything.test_array_int_none_out(), [1, 2, 3, 4, 5])
+
+    def test_array_int_inout(self):
         self.assertEqual(Everything.test_array_int_inout([1, 5, 42, -8]), [6, 43, -7])
 
+    def test_array_int_inout_empty(self):
+        self.assertEqual(Everything.test_array_int_inout([]), [])
+
+    def test_array_gint8_in(self):
         if sys.version_info >= (3, 0):
             self.assertEqual(Everything.test_array_gint8_in(b'\x01\x03\x05'), 9)
         self.assertEqual(Everything.test_array_gint8_in([1, 3, 5, -50]), -41)
+
+    def test_array_gint16_in(self):
         self.assertEqual(Everything.test_array_gint16_in([256, 257, -1000, 10000]), 9513)
+
+    def test_array_gint32_in(self):
         self.assertEqual(Everything.test_array_gint32_in([30000, 1, -2]), 29999)
+
+    def test_array_gint64_in(self):
         self.assertEqual(Everything.test_array_gint64_in([2 ** 33, 2 ** 34]), 2 ** 33 + 2 ** 34)
 
+    def test_array_gtype_in(self):
         self.assertEqual(Everything.test_array_gtype_in(
             [GObject.TYPE_STRING, GObject.TYPE_UINT64, GObject.TYPE_VARIANT]),
             '[gchararray,guint64,GVariant,]')
 
-    def test_array_fixed_size(self):
+    def test_array_fixed_size_int_in(self):
         # fixed length of 5
         self.assertEqual(Everything.test_array_fixed_size_int_in([1, 2, -10, 5, 3]), 1)
+
+    def test_array_fixed_size_int_in_error(self):
         self.assertRaises(ValueError, Everything.test_array_fixed_size_int_in, [1, 2, 3, 4])
         self.assertRaises(ValueError, Everything.test_array_fixed_size_int_in, [1, 2, 3, 4, 5, 6])
 
+    def test_array_fixed_size_int_out(self):
         self.assertEqual(Everything.test_array_fixed_size_int_out(), [0, 1, 2, 3, 4])
+
+    def test_array_fixed_size_int_return(self):
         self.assertEqual(Everything.test_array_fixed_size_int_return(), [0, 1, 2, 3, 4])
 
-    def test_ptrarray(self):
-        # transfer container
+    def test_garray_container_return(self):
+        # GPtrArray transfer container
         result = Everything.test_garray_container_return()
         self.assertEqual(result, ['regress'])
         result = None
 
-        # transfer full
+    def test_garray_full_return(self):
+        # GPtrArray transfer full
         result = Everything.test_garray_full_return()
         self.assertEqual(result, ['regress'])
         result = None
 
-    def test_strv(self):
+    def test_strv_out(self):
         self.assertEqual(Everything.test_strv_out(), ['thanks', 'for', 'all', 'the', 'fish'])
+
+    def test_strv_out_c(self):
         self.assertEqual(Everything.test_strv_out_c(), ['thanks', 'for', 'all', 'the', 'fish'])
+
+    def test_strv_out_container(self):
         self.assertEqual(Everything.test_strv_out_container(), ['1', '2', '3'])
+
+    def test_strv_outarg(self):
         self.assertEqual(Everything.test_strv_outarg(), ['1', '2', '3'])
 
+    def test_strv_in_gvalue(self):
         self.assertEqual(Everything.test_strv_in_gvalue(), ['one', 'two', 'three'])
 
+    def test_strv_in(self):
         Everything.test_strv_in(['1', '2', '3'])
 
     def test_glist(self):
